==> apache-camel-4.8.9-src.zip: apache-camel-4.8.9/core/camel-core-model/src/main/java/org/apache/camel/model/CircuitBreakerDefinition.java <==
/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.camel.model;

import java.util.Iterator;
import java.util.List;

import jakarta.xml.bind.annotation.XmlAccessType;
import jakarta.xml.bind.annotation.XmlAccessorType;
import jakarta.xml.bind.annotation.XmlAttribute;
import jakarta.xml.bind.annotation.XmlElement;
import jakarta.xml.bind.annotation.XmlElementRef;
import jakarta.xml.bind.annotation.XmlRootElement;

import org.apache.camel.spi.Metadata;

/**
 * Route messages in a fault tolerance way using Circuit Breaker
 */
@Metadata(label = "eip,routing,error")
@XmlRootElement(name = "circuitBreaker")
@XmlAccessorType(XmlAccessType.FIELD)
public class CircuitBreakerDefinition extends OutputDefinition<CircuitBreakerDefinition> {

    @XmlAttribute
    private String configuration;
    @XmlElement
    private Resilience4jConfigurationDefinition resilience4jConfiguration;
    @XmlElement
    private FaultToleranceConfigurationDefinition faultToleranceConfiguration;
    @XmlElement
    private OnFallbackDefinition onFallback;

    public CircuitBreakerDefinition() {
    }

    protected CircuitBreakerDefinition(CircuitBreakerDefinition source) {
        super(source);
        this.configuration = source.configuration;
        this.resilience4jConfiguration
                = source.resilience4jConfiguration != null ? source.resilience4jConfiguration.copyDefinition() : null;
        this.faultToleranceConfiguration
                = source.faultToleranceConfiguration != null ? source.faultToleranceConfiguration.copyDefinition() : null;
        this.onFallback = source.onFallback != null ? source.onFallback.copyDefinition() : null;
    }

    @Override
    public CircuitBreakerDefinition copyDefinition() {
        return new CircuitBreakerDefinition(this);
    }

    @Override
    public String toString() {
        return "CircuitBreaker[" + getOutputs() + "]";
    }

    @Override
    public String getShortName() {
        return "circuitBreaker";
    }

    @Override
    public String getLabel() {
        return "circuitBreaker";
    }

    @Override
    public List<ProcessorDefinition<?>> getOutputs() {
        return outputs;
    }

    @XmlElementRef
    @Override
    public void setOutputs(List<ProcessorDefinition<?>> outputs) {
        super.setOutputs(outputs);
    }

    @Override
    public void addOutput(ProcessorDefinition<?> output) {
        if (output instanceof OnFallbackDefinition onFallbackDefinition) {
            onFallback = onFallbackDefinition;
        } else {
            if (onFallback != null) {
                onFallback.addOutput(output);
            } else {
                super.addOutput(output);
            }
        }
    }

    @Override
    public ProcessorDefinition<?> end() {
        if (onFallback != null) {
            // end fallback as well
            onFallback.end();
        }
        return super.end();
    }

    @Override
    public void preCreateProcessor() {
        // move the fallback from outputs to fallback which we need to ensure
        // such as when using the XML DSL
        Iterator<ProcessorDefinition<?>> it = outputs.iterator();
        while (it.hasNext()) {
            ProcessorDefinition<?> out = it.next();
            if (out instanceof OnFallbackDefinition onFallbackDefinition) {
                onFallback = onFallbackDefinition;
                it.remove();
            }
        }
    }

    public Resilience4jConfigurationDefinition getResilience4jConfiguration() {
        return resilience4jConfiguration;
    }

    public void setResilience4jConfiguration(Resilience4jConfigurationDefinition resilience4jConfiguration) {
        this.resilience4jConfiguration = resilience4jConfiguration;
    }

    public FaultToleranceConfigurationDefinition getFaultToleranceConfiguration() {
        return faultToleranceConfiguration;
    }

    public void setFaultToleranceConfiguration(FaultToleranceConfigurationDefinition faultToleranceConfiguration) {
        this.faultToleranceConfiguration = faultToleranceConfiguration;
    }

    public String getConfiguration() {
        return configuration;
    }

    /**
     * Refers to a circuit breaker configuration (such as resillience4j, or microprofile-fault-tolerance) to use for
     * configuring the circuit breaker EIP.
     */
    public void setConfiguration(String configuration) {
        this.configuration = configuration;
    }

    public OnFallbackDefinition getOnFallback() {
        return onFallback;
    }

    public void setOnFallback(OnFallbackDefinition onFallback) {
        this.onFallback = onFallback;
    }

    // Fluent API
    // -------------------------------------------------------------------------
    /**
     * Configures the circuit breaker to use Resilience4j.
     * <p/>
     * Use <tt>end</tt> when configuration is complete, to return back to the Circuit Breaker EIP.
     */
    public Resilience4jConfigurationDefinition resilience4jConfiguration() {
        resilience4jConfiguration
                = resilience4jConfiguration == null ? new Resilience4jConfigurationDefinition(this) : resilience4jConfiguration;
        return resilience4jConfiguration;
    }

    /**
     * Configures the circuit breaker to use Resilience4j with the given configuration.
     */
    public CircuitBreakerDefinition resilience4jConfiguration(Resilience4jConfigurationDefinition configuration) {
        resilience4jConfiguration = configuration;
        return this;
    }

    /**
     * Configures the circuit breaker to use MicroProfile Fault Tolerance.
     * <p/>
     * Use <tt>end</tt> when configuration is complete, to return back to the Circuit Breaker EIP.
     */
    public FaultToleranceConfigurationDefinition faultToleranceConfiguration() {
        faultToleranceConfiguration = faultToleranceConfiguration == null
                ? new FaultToleranceConfigurationDefinition(this) : faultToleranceConfiguration;
        return faultToleranceConfiguration;
    }

    /**
     * Configures the circuit breaker to use MicroProfile Fault Tolerance with the given configuration.
     */
    public CircuitBreakerDefinition faultToleranceConfiguration(FaultToleranceConfigurationDefinition configuration) {
        faultToleranceConfiguration = configuration;
        return this;
    }

    /**
     * Refers to a configuration to use for configuring the circuit breaker.
     */
    public CircuitBreakerDefinition configuration(String ref) {
        configuration = ref;
        return this;
    }

    /**
     * The fallback route path to execute that does <b>not</b> go over the network.
     * <p>
     * This should be a static or cached result that can immediately be returned upon failure. If the fallback requires
     * network connection then use {@link #onFallbackViaNetwork()}.
     */
    public CircuitBreakerDefinition onFallback() {
        onFallback = new OnFallbackDefinition();
        onFallback.setParent(this);
        return this;
    }

    /**
     * The fallback route path to execute that will go over the network.
     * <p/>
     * If the fallback will go over the network it is another possible point of failure.
     */
    public CircuitBreakerDefinition onFallbackViaNetwork() {
        onFallback = new OnFallbackDefinition();
        onFallback.setFallbackViaNetwork(Boolean.toString(true));
        onFallback.setParent(this);
        return this;
    }

}

==> apache-camel-4.14.4-src.zip: apache-camel-4.14.4/core/camel-core-model/src/main/java/org/apache/camel/model/CircuitBreakerDefinition.java <==
/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.camel.model;

import java.util.List;

import jakarta.xml.bind.annotation.XmlAccessType;
import jakarta.xml.bind.annotation.XmlAccessorType;
import jakarta.xml.bind.annotation.XmlAttribute;
import jakarta.xml.bind.annotation.XmlElement;
import jakarta.xml.bind.annotation.XmlElementRef;
import jakarta.xml.bind.annotation.XmlRootElement;
import jakarta.xml.bind.annotation.XmlType;

import org.apache.camel.spi.Metadata;

/**
 * Route messages in a fault tolerance way using Circuit Breaker
 */
@Metadata(label = "eip,routing,error")
@XmlRootElement(name = "circuitBreaker")
@XmlAccessorType(XmlAccessType.FIELD)
@XmlType(propOrder = { "resilience4jConfiguration", "faultToleranceConfiguration", "outputs", "onFallback" })
public class CircuitBreakerDefinition extends OutputDefinition<CircuitBreakerDefinition> {

    @XmlAttribute
    private String configuration;
    @XmlAttribute
    @Metadata(label = "advanced", javaType = "java.lang.Boolean", defaultValue = "false")
    private Boolean inheritErrorHandler;
    @XmlElement
    private Resilience4jConfigurationDefinition resilience4jConfiguration;
    @XmlElement
    private FaultToleranceConfigurationDefinition faultToleranceConfiguration;
    @XmlElement
    private OnFallbackDefinition onFallback;

    public CircuitBreakerDefinition() {
    }

    protected CircuitBreakerDefinition(CircuitBreakerDefinition source) {
        super(source);
        this.configuration = source.configuration;
        this.inheritErrorHandler = source.inheritErrorHandler;
        this.resilience4jConfiguration
                = source.resilience4jConfiguration != null ? source.resilience4jConfiguration.copyDefinition() : null;
        this.faultToleranceConfiguration
                = source.faultToleranceConfiguration != null ? source.faultToleranceConfiguration.copyDefinition() : null;
        this.onFallback = source.onFallback != null ? source.onFallback.copyDefinition() : null;
    }

    @Override
    public CircuitBreakerDefinition copyDefinition() {
        return new CircuitBreakerDefinition(this);
    }

    @Override
    public String toString() {
        return "CircuitBreaker[" + getOutputs() + "]";
    }

    @Override
    public String getShortName() {
        return "circuitBreaker";
    }

    @Override
    public String getLabel() {
        return "circuitBreaker";
    }

    @Override
    public List<ProcessorDefinition<?>> getOutputs() {
        return outputs;
    }

    @XmlElementRef
    @Override
    public void setOutputs(List<ProcessorDefinition<?>> outputs) {
        super.setOutputs(outputs);
    }

    @Override
    public void addOutput(ProcessorDefinition<?> output) {
        if (onFallback != null) {
            onFallback.addOutput(output);
        } else {
            super.addOutput(output);
        }
    }

    public Resilience4jConfigurationDefinition getResilience4jConfiguration() {
        return resilience4jConfiguration;
    }

    public void setResilience4jConfiguration(Resilience4jConfigurationDefinition resilience4jConfiguration) {
        this.resilience4jConfiguration = resilience4jConfiguration;
    }

    public FaultToleranceConfigurationDefinition getFaultToleranceConfiguration() {
        return faultToleranceConfiguration;
    }

    public void setFaultToleranceConfiguration(FaultToleranceConfigurationDefinition faultToleranceConfiguration) {
        this.faultToleranceConfiguration = faultToleranceConfiguration;
    }

    public String getConfiguration() {
        return configuration;
    }

    /**
     * Refers to a circuit breaker configuration (such as resillience4j, or microprofile-fault-tolerance) to use for
     * configuring the circuit breaker EIP.
     */
    public void setConfiguration(String configuration) {
        this.configuration = configuration;
    }

    @Override
    public Boolean getInheritErrorHandler() {
        return inheritErrorHandler;
    }

    @Override
    public void setInheritErrorHandler(Boolean inheritErrorHandler) {
        this.inheritErrorHandler = inheritErrorHandler;
    }

    public OnFallbackDefinition getOnFallback() {
        return onFallback;
    }

    public void setOnFallback(OnFallbackDefinition onFallback) {
        this.onFallback = onFallback;
    }

    // Fluent API
    // -------------------------------------------------------------------------
    /**
     * Configures the circuit breaker to use Resilience4j.
     * <p/>
     * Use <tt>end</tt> when configuration is complete, to return back to the Circuit Breaker EIP.
     */
    public Resilience4jConfigurationDefinition resilience4jConfiguration() {
        resilience4jConfiguration
                = resilience4jConfiguration == null ? new Resilience4jConfigurationDefinition(this) : resilience4jConfiguration;
        return resilience4jConfiguration;
    }

    /**
     * Configures the circuit breaker to use Resilience4j with the given configuration.
     */
    public CircuitBreakerDefinition resilience4jConfiguration(Resilience4jConfigurationDefinition configuration) {
        resilience4jConfiguration = configuration;
        return this;
    }

    /**
     * Configures the circuit breaker to use MicroProfile Fault Tolerance.
     * <p/>
     * Use <tt>end</tt> when configuration is complete, to return back to the Circuit Breaker EIP.
     */
    public FaultToleranceConfigurationDefinition faultToleranceConfiguration() {
        faultToleranceConfiguration = faultToleranceConfiguration == null
                ? new FaultToleranceConfigurationDefinition(this) : faultToleranceConfiguration;
        return faultToleranceConfiguration;
    }

    /**
     * Configures the circuit breaker to use MicroProfile Fault Tolerance with the given configuration.
     */
    public CircuitBreakerDefinition faultToleranceConfiguration(FaultToleranceConfigurationDefinition configuration) {
        faultToleranceConfiguration = configuration;
        return this;
    }

    /**
     * Refers to a configuration to use for configuring the circuit breaker.
     */
    public CircuitBreakerDefinition configuration(String ref) {
        configuration = ref;
        return this;
    }

    /**
     * To turn on or off Camel error handling during circuit breaker.
     *
     * If this is enabled then Camel error handler will first trigger if there is an error in the circuit breaker, which
     * allows to let Camel handle redeliveries. If all attempts is failed, then after the circuit breaker is finished,
     * then Camel error handler can handle the error as well such as the dead letter channel.
     *
     * By default, Camel error handler is turned off.
     */
    public CircuitBreakerDefinition inheritErrorHandler(boolean inheritErrorHandler) {
        this.inheritErrorHandler = inheritErrorHandler;
        return this;
    }

    /**
     * The fallback route path to execute that does <b>not</b> go over the network.
     * <p>
     * This should be a static or cached result that can immediately be returned upon failure. If the fallback requires
     * network connection then use {@link #onFallbackViaNetwork()}.
     */
    public CircuitBreakerDefinition onFallback() {
        onFallback = new OnFallbackDefinition();
        onFallback.setParent(this);
        return this;
    }

    /**
     * The fallback route path to execute that will go over the network.
     * <p/>
     * If the fallback will go over the network it is another possible point of failure.
     */
    public CircuitBreakerDefinition onFallbackViaNetwork() {
        onFallback = new OnFallbackDefinition();
        onFallback.setFallbackViaNetwork(Boolean.toString(true));
        onFallback.setParent(this);
        return this;
    }

}
